# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from .services.budget_service import BudgetServiceClient
from .services.budget_service import BudgetServiceAsyncClient

from .types.budget_model import AllUpdatesRule
from .types.budget_model import Budget
from .types.budget_model import BudgetAmount
from .types.budget_model import CustomPeriod
from .types.budget_model import Filter
from .types.budget_model import LastPeriodAmount
from .types.budget_model import ThresholdRule
from .types.budget_model import CalendarPeriod
from .types.budget_service import CreateBudgetRequest
from .types.budget_service import DeleteBudgetRequest
from .types.budget_service import GetBudgetRequest
from .types.budget_service import ListBudgetsRequest
from .types.budget_service import ListBudgetsResponse
from .types.budget_service import UpdateBudgetRequest

__all__ = (
    "BudgetServiceAsyncClient",
    "AllUpdatesRule",
    "Budget",
    "BudgetAmount",
    "BudgetServiceClient",
    "CalendarPeriod",
    "CreateBudgetRequest",
    "CustomPeriod",
    "DeleteBudgetRequest",
    "Filter",
    "GetBudgetRequest",
    "LastPeriodAmount",
    "ListBudgetsRequest",
    "ListBudgetsResponse",
    "ThresholdRule",
    "UpdateBudgetRequest",
)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.billing.budgets_v1beta1.types import budget_model
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.billing.budgets.v1beta1",
    manifest={
        "CreateBudgetRequest",
        "UpdateBudgetRequest",
        "GetBudgetRequest",
        "ListBudgetsRequest",
        "ListBudgetsResponse",
        "DeleteBudgetRequest",
    },
)


class CreateBudgetRequest(proto.Message):
    r"""Request for CreateBudget
    Attributes:
        parent (str):
            Required. The name of the billing account to create the
            budget in. Values are of the form
            ``billingAccounts/{billingAccountId}``.
        budget (google.cloud.billing.budgets_v1beta1.types.Budget):
            Required. Budget to create.
    """

    parent = proto.Field(proto.STRING, number=1,)
    budget = proto.Field(proto.MESSAGE, number=2, message=budget_model.Budget,)


class UpdateBudgetRequest(proto.Message):
    r"""Request for UpdateBudget
    Attributes:
        budget (google.cloud.billing.budgets_v1beta1.types.Budget):
            Required. The updated budget object.
            The budget to update is specified by the budget
            name in the budget.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Optional. Indicates which fields in the provided budget to
            update. Read-only fields (such as ``name``) cannot be
            changed. If this is not provided, then only fields with
            non-default values from the request are updated. See
            https://developers.google.com/protocol-buffers/docs/proto3#default
            for more details about default values.
    """

    budget = proto.Field(proto.MESSAGE, number=1, message=budget_model.Budget,)
    update_mask = proto.Field(
        proto.MESSAGE, number=2, message=field_mask_pb2.FieldMask,
    )


class GetBudgetRequest(proto.Message):
    r"""Request for GetBudget
    Attributes:
        name (str):
            Required. Name of budget to get. Values are of the form
            ``billingAccounts/{billingAccountId}/budgets/{budgetId}``.
    """

    name = proto.Field(proto.STRING, number=1,)


class ListBudgetsRequest(proto.Message):
    r"""Request for ListBudgets
    Attributes:
        parent (str):
            Required. Name of billing account to list budgets under.
            Values are of the form
            ``billingAccounts/{billingAccountId}``.
        page_size (int):
            Optional. The maximum number of budgets to
            return per page. The default and maximum value
            are 100.
        page_token (str):
            Optional. The value returned by the last
            ``ListBudgetsResponse`` which indicates that this is a
            continuation of a prior ``ListBudgets`` call, and that the
            system should return the next page of data.
    """

    parent = proto.Field(proto.STRING, number=1,)
    page_size = proto.Field(proto.INT32, number=2,)
    page_token = proto.Field(proto.STRING, number=3,)


class ListBudgetsResponse(proto.Message):
    r"""Response for ListBudgets
    Attributes:
        budgets (Sequence[google.cloud.billing.budgets_v1beta1.types.Budget]):
            List of the budgets owned by the requested
            billing account.
        next_page_token (str):
            If not empty, indicates that there may be more budgets that
            match the request; this value should be passed in a new
            ``ListBudgetsRequest``.
    """

    @property
    def raw_page(self):
        return self

    budgets = proto.RepeatedField(proto.MESSAGE, number=1, message=budget_model.Budget,)
    next_page_token = proto.Field(proto.STRING, number=2,)


class DeleteBudgetRequest(proto.Message):
    r"""Request for DeleteBudget
    Attributes:
        name (str):
            Required. Name of the budget to delete. Values are of the
            form
            ``billingAccounts/{billingAccountId}/budgets/{budgetId}``.
    """

    name = proto.Field(proto.STRING, number=1,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.protobuf import struct_pb2  # type: ignore
from google.type import date_pb2  # type: ignore
from google.type import money_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.billing.budgets.v1beta1",
    manifest={
        "CalendarPeriod",
        "Budget",
        "BudgetAmount",
        "LastPeriodAmount",
        "ThresholdRule",
        "AllUpdatesRule",
        "Filter",
        "CustomPeriod",
    },
)


class CalendarPeriod(proto.Enum):
    r"""A ``CalendarPeriod`` represents the abstract concept of a time
    period that has a canonical start. Grammatically, "the start of the
    current ``CalendarPeriod``". All calendar times begin at 12 AM US
    and Canadian Pacific Time (UTC-8).
    """
    CALENDAR_PERIOD_UNSPECIFIED = 0
    MONTH = 1
    QUARTER = 2
    YEAR = 3


class Budget(proto.Message):
    r"""A budget is a plan that describes what you expect to spend on
    Cloud projects, plus the rules to execute as spend is tracked
    against that plan, (for example, send an alert when 90% of the
    target spend is met). The budget time period is configurable,
    with options such as month (default), quarter, year, or custom
    time period.

    Attributes:
        name (str):
            Output only. Resource name of the budget. The resource name
            implies the scope of a budget. Values are of the form
            ``billingAccounts/{billingAccountId}/budgets/{budgetId}``.
        display_name (str):
            User data for display name in UI.
            Validation: <= 60 chars.
        budget_filter (google.cloud.billing.budgets_v1beta1.types.Filter):
            Optional. Filters that define which resources
            are used to compute the actual spend against the
            budget amount, such as projects, services, and
            the budget's time period, as well as other
            filters.
        amount (google.cloud.billing.budgets_v1beta1.types.BudgetAmount):
            Required. Budgeted amount.
        threshold_rules (Sequence[google.cloud.billing.budgets_v1beta1.types.ThresholdRule]):
            Optional. Rules that trigger alerts
            (notifications of thresholds being crossed) when
            spend exceeds the specified percentages of the
            budget.
        all_updates_rule (google.cloud.billing.budgets_v1beta1.types.AllUpdatesRule):
            Optional. Rules to apply to notifications
            sent based on budget spend and thresholds.
        etag (str):
            Optional. Etag to validate that the object is
            unchanged for a read-modify-write operation.
            An empty etag will cause an update to overwrite
            other changes.
    """

    name = proto.Field(proto.STRING, number=1,)
    display_name = proto.Field(proto.STRING, number=2,)
    budget_filter = proto.Field(proto.MESSAGE, number=3, message="Filter",)
    amount = proto.Field(proto.MESSAGE, number=4, message="BudgetAmount",)
    threshold_rules = proto.RepeatedField(
        proto.MESSAGE, number=5, message="ThresholdRule",
    )
    all_updates_rule = proto.Field(proto.MESSAGE, number=6, message="AllUpdatesRule",)
    etag = proto.Field(proto.STRING, number=7,)


class BudgetAmount(proto.Message):
    r"""The budgeted amount for each usage period.
    Attributes:
        specified_amount (google.type.money_pb2.Money):
            A specified amount to use as the budget. ``currency_code``
            is optional. If specified when creating a budget, it must
            match the currency of the billing account. If specified when
            updating a budget, it must match the currency_code of the
            existing budget. The ``currency_code`` is provided on
            output.
        last_period_amount (google.cloud.billing.budgets_v1beta1.types.LastPeriodAmount):
            Use the last period's actual spend as the budget for the
            present period. LastPeriodAmount can only be set when the
            budget's time period is a
            [Filter.calendar_period][google.cloud.billing.budgets.v1beta1.Filter.calendar_period].
            It cannot be set in combination with
            [Filter.custom_period][google.cloud.billing.budgets.v1beta1.Filter.custom_period].
    """

    specified_amount = proto.Field(
        proto.MESSAGE, number=1, oneof="budget_amount", message=money_pb2.Money,
    )
    last_period_amount = proto.Field(
        proto.MESSAGE, number=2, oneof="budget_amount", message="LastPeriodAmount",
    )


class LastPeriodAmount(proto.Message):
    r"""Describes a budget amount targeted to the last
    [Filter.calendar_period][google.cloud.billing.budgets.v1beta1.Filter.calendar_period]
    spend. At this time, the amount is automatically 100% of the last
    calendar period's spend; that is, there are no other options yet.
    Future configuration options will be described here (for example,
    configuring a percentage of last period's spend). LastPeriodAmount
    cannot be set for a budget configured with a
    [Filter.custom_period][google.cloud.billing.budgets.v1beta1.Filter.custom_period].
        """


class ThresholdRule(proto.Message):
    r"""ThresholdRule contains a definition of a threshold which triggers an
    alert (a notification of a threshold being crossed) to be sent when
    spend goes above the specified amount. Alerts are automatically
    e-mailed to users with the Billing Account Administrator role or the
    Billing Account User role. The thresholds here have no effect on
    notifications sent to anything configured under
    ``Budget.all_updates_rule``.

    Attributes:
        threshold_percent (float):
            Required. Send an alert when this threshold
            is exceeded. This is a 1.0-based percentage, so
            0.5 = 50%. Validation: non-negative number.
        spend_basis (google.cloud.billing.budgets_v1beta1.types.ThresholdRule.Basis):
            Optional. The type of basis used to determine if spend has
            passed the threshold. Behavior defaults to CURRENT_SPEND if
            not set.
    """

    class Basis(proto.Enum):
        r"""The type of basis used to determine if spend has passed the
        threshold.
        """
        BASIS_UNSPECIFIED = 0
        CURRENT_SPEND = 1
        FORECASTED_SPEND = 2

    threshold_percent = proto.Field(proto.DOUBLE, number=1,)
    spend_basis = proto.Field(proto.ENUM, number=2, enum=Basis,)


class AllUpdatesRule(proto.Message):
    r"""AllUpdatesRule defines notifications that are sent based on
    budget spend and thresholds.

    Attributes:
        pubsub_topic (str):
            Optional. The name of the Pub/Sub topic where budget related
            messages will be published, in the form
            ``projects/{project_id}/topics/{topic_id}``. Updates are
            sent at regular intervals to the topic. The topic needs to
            be created before the budget is created; see
            https://cloud.google.com/billing/docs/how-to/budgets-programmatic-notifications
            for more details. Caller is expected to have
            ``pubsub.topics.setIamPolicy`` permission on the topic when
            it's set for a budget, otherwise, the API call will fail
            with PERMISSION_DENIED. See
            https://cloud.google.com/billing/docs/how-to/budgets-programmatic-notifications#permissions_required_for_this_task
            for more details on Pub/Sub roles and permissions.
        schema_version (str):
            Optional. Required when
            [AllUpdatesRule.pubsub_topic][google.cloud.billing.budgets.v1beta1.AllUpdatesRule.pubsub_topic]
            is set. The schema version of the notification sent to
            [AllUpdatesRule.pubsub_topic][google.cloud.billing.budgets.v1beta1.AllUpdatesRule.pubsub_topic].
            Only "1.0" is accepted. It represents the JSON schema as
            defined in
            https://cloud.google.com/billing/docs/how-to/budgets-programmatic-notifications#notification_format.
        monitoring_notification_channels (Sequence[str]):
            Optional. Targets to send notifications to when a threshold
            is exceeded. This is in addition to default recipients who
            have billing account IAM roles. The value is the full REST
            resource name of a monitoring notification channel with the
            form
            ``projects/{project_id}/notificationChannels/{channel_id}``.
            A maximum of 5 channels are allowed. See
            https://cloud.google.com/billing/docs/how-to/budgets-notification-recipients
            for more details.
        disable_default_iam_recipients (bool):
            Optional. When set to true, disables default
            notifications sent when a threshold is exceeded.
            Default notifications are sent to those with
            Billing Account Administrator and Billing
            Account User IAM roles for the target account.
    """

    pubsub_topic = proto.Field(proto.STRING, number=1,)
    schema_version = proto.Field(proto.STRING, number=2,)
    monitoring_notification_channels = proto.RepeatedField(proto.STRING, number=3,)
    disable_default_iam_recipients = proto.Field(proto.BOOL, number=4,)


class Filter(proto.Message):
    r"""A filter for a budget, limiting the scope of the cost to
    calculate.

    Attributes:
        projects (Sequence[str]):
            Optional. A set of projects of the form
            ``projects/{project}``, specifying that usage from only this
            set of projects should be included in the budget. If
            omitted, the report will include all usage for the billing
            account, regardless of which project the usage occurred on.
            Only zero or one project can be specified currently.
        credit_types (Sequence[str]):
            Optional. If
            [Filter.credit_types_treatment][google.cloud.billing.budgets.v1beta1.Filter.credit_types_treatment]
            is INCLUDE_SPECIFIED_CREDITS, this is a list of credit types
            to be subtracted from gross cost to determine the spend for
            threshold calculations. See `a list of acceptable credit
            type
            values <https://cloud.google.com/billing/docs/how-to/export-data-bigquery-tables#credits-type>`__.

            If
            [Filter.credit_types_treatment][google.cloud.billing.budgets.v1beta1.Filter.credit_types_treatment]
            is **not** INCLUDE_SPECIFIED_CREDITS, this field must be
            empty.
        credit_types_treatment (google.cloud.billing.budgets_v1beta1.types.Filter.CreditTypesTreatment):
            Optional. If not set, default behavior is
            ``INCLUDE_ALL_CREDITS``.
        services (Sequence[str]):
            Optional. A set of services of the form
            ``services/{service_id}``, specifying that usage from only
            this set of services should be included in the budget. If
            omitted, the report will include usage for all the services.
            The service names are available through the Catalog API:
            https://cloud.google.com/billing/v1/how-tos/catalog-api.
        subaccounts (Sequence[str]):
            Optional. A set of subaccounts of the form
            ``billingAccounts/{account_id}``, specifying that usage from
            only this set of subaccounts should be included in the
            budget. If a subaccount is set to the name of the parent
            account, usage from the parent account will be included. If
            omitted, the report will include usage from the parent
            account and all subaccounts, if they exist.
        labels (Sequence[google.cloud.billing.budgets_v1beta1.types.Filter.LabelsEntry]):
            Optional. A single label and value pair
            specifying that usage from only this set of
            labeled resources should be included in the
            budget. Currently, multiple entries or multiple
            values per entry are not allowed. If omitted,
            the report will include all labeled and
            unlabeled usage.
        calendar_period (google.cloud.billing.budgets_v1beta1.types.CalendarPeriod):
            Optional. Specifies to track usage for
            recurring calendar period. For example, assume
            that CalendarPeriod.QUARTER is set. The budget
            will track usage from April 1 to June 30, when
            the current calendar month is April, May, June.
            After that, it will track usage from July 1 to
            September 30 when the current calendar month is
            July, August, September, so on.
        custom_period (google.cloud.billing.budgets_v1beta1.types.CustomPeriod):
            Optional. Specifies to track usage from any
            start date (required) to any end date
            (optional). This time period is static, it does
            not recur.
    """

    class CreditTypesTreatment(proto.Enum):
        r"""Specifies how credits are applied when determining the spend for
        threshold calculations. Budgets track the total cost minus any
        applicable selected credits. `See the documentation for a list of
        credit
        types <https://cloud.google.com/billing/docs/how-to/export-data-bigquery-tables#credits-type>`__.
        """
        CREDIT_TYPES_TREATMENT_UNSPECIFIED = 0
        INCLUDE_ALL_CREDITS = 1
        EXCLUDE_ALL_CREDITS = 2
        INCLUDE_SPECIFIED_CREDITS = 3

    projects = proto.RepeatedField(proto.STRING, number=1,)
    credit_types = proto.RepeatedField(proto.STRING, number=7,)
    credit_types_treatment = proto.Field(
        proto.ENUM, number=4, enum=CreditTypesTreatment,
    )
    services = proto.RepeatedField(proto.STRING, number=3,)
    subaccounts = proto.RepeatedField(proto.STRING, number=5,)
    labels = proto.MapField(
        proto.STRING, proto.MESSAGE, number=6, message=struct_pb2.ListValue,
    )
    calendar_period = proto.Field(
        proto.ENUM, number=8, oneof="usage_period", enum="CalendarPeriod",
    )
    custom_period = proto.Field(
        proto.MESSAGE, number=9, oneof="usage_period", message="CustomPeriod",
    )


class CustomPeriod(proto.Message):
    r"""All date times begin at 12 AM US and Canadian Pacific Time
    (UTC-8).

    Attributes:
        start_date (google.type.date_pb2.Date):
            Required. The start date must be after
            January 1, 2017.
        end_date (google.type.date_pb2.Date):
            Optional. The end date of the time period. Budgets with
            elapsed end date won't be processed. If unset, specifies to
            track all usage incurred since the start_date.
    """

    start_date = proto.Field(proto.MESSAGE, number=1, message=date_pb2.Date,)
    end_date = proto.Field(proto.MESSAGE, number=2, message=date_pb2.Date,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import Dict, Sequence, Tuple, Type, Union
import pkg_resources

import google.api_core.client_options as ClientOptions  # type: ignore
from google.api_core import exceptions as core_exceptions  # type: ignore
from google.api_core import gapic_v1  # type: ignore
from google.api_core import retry as retries  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.billing.budgets_v1beta1.services.budget_service import pagers
from google.cloud.billing.budgets_v1beta1.types import budget_model
from google.cloud.billing.budgets_v1beta1.types import budget_service
from .transports.base import BudgetServiceTransport, DEFAULT_CLIENT_INFO
from .transports.grpc_asyncio import BudgetServiceGrpcAsyncIOTransport
from .client import BudgetServiceClient


class BudgetServiceAsyncClient:
    """BudgetService stores Cloud Billing budgets, which define a
    budget plan and rules to execute as we track spend against that
    plan.
    """

    _client: BudgetServiceClient

    DEFAULT_ENDPOINT = BudgetServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = BudgetServiceClient.DEFAULT_MTLS_ENDPOINT

    budget_path = staticmethod(BudgetServiceClient.budget_path)
    parse_budget_path = staticmethod(BudgetServiceClient.parse_budget_path)
    common_billing_account_path = staticmethod(
        BudgetServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        BudgetServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(BudgetServiceClient.common_folder_path)
    parse_common_folder_path = staticmethod(
        BudgetServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        BudgetServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        BudgetServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(BudgetServiceClient.common_project_path)
    parse_common_project_path = staticmethod(
        BudgetServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(BudgetServiceClient.common_location_path)
    parse_common_location_path = staticmethod(
        BudgetServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            BudgetServiceAsyncClient: The constructed client.
        """
        return BudgetServiceClient.from_service_account_info.__func__(BudgetServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            BudgetServiceAsyncClient: The constructed client.
        """
        return BudgetServiceClient.from_service_account_file.__func__(BudgetServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @property
    def transport(self) -> BudgetServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            BudgetServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(BudgetServiceClient).get_transport_class, type(BudgetServiceClient)
    )

    def __init__(
        self,
        *,
        credentials: ga_credentials.Credentials = None,
        transport: Union[str, BudgetServiceTransport] = "grpc_asyncio",
        client_options: ClientOptions = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the budget service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.BudgetServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = BudgetServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def create_budget(
        self,
        request: budget_service.CreateBudgetRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> budget_model.Budget:
        r"""Creates a new budget. See
        <a href="https://cloud.google.com/billing/quotas">Quotas
        and limits</a> for more information on the limits of the
        number of budgets you can create.

        Args:
            request (:class:`google.cloud.billing.budgets_v1beta1.types.CreateBudgetRequest`):
                The request object. Request for CreateBudget
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.billing.budgets_v1beta1.types.Budget:
                A budget is a plan that describes
                what you expect to spend on Cloud
                projects, plus the rules to execute as
                spend is tracked against that plan, (for
                example, send an alert when 90% of the
                target spend is met). The budget time
                period is configurable, with options
                such as month (default), quarter, year,
                or custom time period.

        """
        # Create or coerce a protobuf request object.
        request = budget_service.CreateBudgetRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_budget,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def update_budget(
        self,
        request: budget_service.UpdateBudgetRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> budget_model.Budget:
        r"""Updates a budget and returns the updated budget.
        WARNING: There are some fields exposed on the Google
        Cloud Console that aren't available on this API. Budget
        fields that are not exposed in this API will not be
        changed by this method.

        Args:
            request (:class:`google.cloud.billing.budgets_v1beta1.types.UpdateBudgetRequest`):
                The request object. Request for UpdateBudget
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.billing.budgets_v1beta1.types.Budget:
                A budget is a plan that describes
                what you expect to spend on Cloud
                projects, plus the rules to execute as
                spend is tracked against that plan, (for
                example, send an alert when 90% of the
                target spend is met). The budget time
                period is configurable, with options
                such as month (default), quarter, year,
                or custom time period.

        """
        # Create or coerce a protobuf request object.
        request = budget_service.UpdateBudgetRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_budget,
            default_retry=retries.Retry(
                initial=0.1,
                maximum=60.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.DeadlineExceeded,
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("budget.name", request.budget.name),)
            ),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def get_budget(
        self,
        request: budget_service.GetBudgetRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> budget_model.Budget:
        r"""Returns a budget.
        WARNING: There are some fields exposed on the Google
        Cloud Console that aren't available on this API. When
        reading from the API, you will not see these fields in
        the return value, though they may have been set in the
        Cloud Console.

        Args:
            request (:class:`google.cloud.billing.budgets_v1beta1.types.GetBudgetRequest`):
                The request object. Request for GetBudget
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.billing.budgets_v1beta1.types.Budget:
                A budget is a plan that describes
                what you expect to spend on Cloud
                projects, plus the rules to execute as
                spend is tracked against that plan, (for
                example, send an alert when 90% of the
                target spend is met). The budget time
                period is configurable, with options
                such as month (default), quarter, year,
                or custom time period.

        """
        # Create or coerce a protobuf request object.
        request = budget_service.GetBudgetRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_budget,
            default_retry=retries.Retry(
                initial=0.1,
                maximum=60.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.DeadlineExceeded,
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def list_budgets(
        self,
        request: budget_service.ListBudgetsRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListBudgetsAsyncPager:
        r"""Returns a list of budgets for a billing account.
        WARNING: There are some fields exposed on the Google
        Cloud Console that aren't available on this API. When
        reading from the API, you will not see these fields in
        the return value, though they may have been set in the
        Cloud Console.

        Args:
            request (:class:`google.cloud.billing.budgets_v1beta1.types.ListBudgetsRequest`):
                The request object. Request for ListBudgets
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.billing.budgets_v1beta1.services.budget_service.pagers.ListBudgetsAsyncPager:
                Response for ListBudgets
                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        request = budget_service.ListBudgetsRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_budgets,
            default_retry=retries.Retry(
                initial=0.1,
                maximum=60.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.DeadlineExceeded,
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListBudgetsAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def delete_budget(
        self,
        request: budget_service.DeleteBudgetRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Deletes a budget. Returns successfully if already
        deleted.

        Args:
            request (:class:`google.cloud.billing.budgets_v1beta1.types.DeleteBudgetRequest`):
                The request object. Request for DeleteBudget
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        # Create or coerce a protobuf request object.
        request = budget_service.DeleteBudgetRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.delete_budget,
            default_retry=retries.Retry(
                initial=0.1,
                maximum=60.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.DeadlineExceeded,
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        await rpc(
            request, retry=retry, timeout=timeout, metadata=metadata,
        )


try:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution(
            "google-cloud-billing-budgets",
        ).version,
    )
except pkg_resources.DistributionNotFound:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()


__all__ = ("BudgetServiceAsyncClient",)
